"""
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        rename_shading_groups.py
Date:           10/21/14
Description:    This script will rename all shading groups from their surface
                shader node plus "SG" 
Installation:   Place this file in your maya scripts directory
                (e.g. ..\My Documents\maya\scripts) then run the following
                python commands:
                    import rename_shading_groups
                    rename_shading_groups.main()
"""
import maya.cmds as cmds
from sys import stdout


class RenameShadingGroups(object):
    @staticmethod
    def get_shading_groups():
        """Returns a sorted list of shading groups minus the defaults"""
        shading_groups = cmds.ls(type='shadingEngine', long=True)
        shading_groups.sort()

        if 'initialShadingGroup' in shading_groups:
            shading_groups.remove("initialShadingGroup")
        if 'initialParticleSE' in shading_groups:
            shading_groups.remove("initialParticleSE")

        return shading_groups

    @staticmethod
    def get_base_name(long_name):
        """Returns the root name of 'long_name' """
        root_name = (long_name.split(":"))[-1]
        root_name = (root_name.split("|"))[-1]

        return root_name

    def get_shading_group_name(self, shading_group):
        """Gets the shading group name based on the surface shader name"""
        shading_group_name = 'NO_SURFACE_SHADER_SG'
        surface_shader = cmds.listConnections(shading_group + ".surfaceShader",
                                              source=True, destination=False)
        if surface_shader:
            shading_group_name = self.get_base_name(surface_shader[0]) + 'SG'

        return shading_group_name


def main():
    """Main entry point. Executes the script"""
    rsg = RenameShadingGroups()
    counter = 1

    # Temporarily rename each shading group to a unique name
    for current in rsg.get_shading_groups():
        cmds.rename(current, (current + "_TEMPRENAME_" + str(counter)))
        counter += 1

    counter = 0

    # Now rename the shading groups properly
    for current in rsg.get_shading_groups():
        name = rsg.get_shading_group_name(current)
        final_name = cmds.rename(current, name)
        final_name = rsg.get_base_name(final_name)
        if final_name != name:
            print('ERROR: Rename for object ' + current + ' failed.')
            print('   Attempted: ' + name)
            print('   Result: ' + final_name)
            counter += 1

    if counter == 0:
        stdout.write('INFO: All shading groups have been renamed')
    else:
        stdout.write('WARNING: Some shading groups could not be renamed. Check script editor')


if __name__ == '__main__':
    main()
